import es, playerlib, serverlib
import re

# This module provides functions used to calculate or immediately return
# information about the server

# Server variables
def svar(var):
	return str(es.ServerVar(var))

# Commandline variables
# typical commandline: ./srcds_i686 -console -game cstrike +map breakfloor +maxplayers 10 -ticrate 33 -port 28015 -pidfile pid -steam
commandline = svar('eventscripts_cmdline')
def clvar(var):
	p = re.compile('[+-]+'+var+'\s+([a-zA-Z0-9_-]+)\s')
	match = re.search(p,commandline)
	if match:
		return match.group(1)
	else:
		return '["'+var+'" not available]'

# On/Off | True/False | Anyone/Noone - Boolean variables
# these are 1/0 in the config files
# anything other than 0 is considered true
def bool(var,true,false):
	value = svar(var)
	def repl(x):
		return value
	def fun():
		if value == 0:
			re.sub('%s',repl,true)
		else:
			return false
	return fun

# Variables that aren't static for a long time
# They get fetched at every request
def fetch(var):
	def fun():
		return svar(var)
	return fun

# Other cool variables
def region():
	regions = [
		'eastcoast',
		'westcoast',
		'south america',
		'europe',
		'asia',
		'australia',
		'middle east',
		'africa',
		'world',
	]
	try:
		# never go over len(regions-1) -> all the rest is world
		return regions[min(int(svar('sv_region')),len(regions) - 1)]
	except ValueError:
		return '[{REGION} N/A]'

def players(filter):
	def fun():
		return str(len(playerlib.getUseridList(filter)))
	return fun

# Collection of all static variables
details = {
	# commandline variables
	'TICRATE':clvar('ticrate'),
	'MAXPLAYERS':clvar('maxplayers'),
	'PORT':clvar('port'),
	'GAME':es.getgame(),

	# usually static server variables that might be used
	'HOSTNAME':svar('hostname'),
	'CONTACT':svar('sv_contact'),
	'ESVERSION':svar('eventscripts_version'),
	'AIRACCELERATE':svar('sv_airaccelerate'),
	'WATERACCELERATE':svar('sv_wateraccelerate'),
	'C4TIMER':svar('mp_c4timer'),
	'MINRATE':svar('sv_minrate'),
	'MAXRATE':svar('sv_maxrate'),
	'MAXUPDATERATE':svar('sv_maxupdaterate'),
	'MINUPDATERATE':svar('sv_minupdaterate'),
	'SPAWNPROTECTIONTIME':svar('mp_spawnprotectiontime'),
	'FOOTSTEPS':bool('mp_footsteps','on','off')(),
	'AUTOTEAMBALANCE':bool('mp_autoteambalance','on','off')(),
	'FLASHLIGHT':bool('mp_flashlight','on','off')(),
	'ALLTALK':bool('sv_alltalk','anyone','none')(),
	'ALLOWSPEC':bool('mp_allowspectators','is','isn\'t')(),
	'LAN':bool('sv_lan','enabled','disabled')(),
	'VOICE':bool('sv_voiceenable','enabled','disabled')(),

	# changing values -> functions waiting to be executed
	# often changes per map or on other occasions
	'TKPUNISH':bool('mp_tkpunish','will','won\'t'),
	'FRIENDLYFIRE':bool('mp_friendlyfire','on','off'),
	'FRAGLIMIT':bool('mp_fraglimit','maximum %s kills','no maximum nr of kills'),
	'TIMELIMIT':bool('mp_timelimit','%s minutes','no time limit'),
	'CHEATS':bool('sv_cheats','on','off'),
	'GRAVITY':fetch('sv_gravity'),
	'ROUNDTIME':fetch('mp_roundtime'),
	'MAXROUNDS':fetch('mp_maxrounds'),
	'STARTMONEY':fetch('mp_startmoney'),

	# all other cool functions
	'REGION':region(),
	'PLAYERS':players('#all'),
	'BOTS':players('#bot'),
	'SPEC':players('#spec'),
	'CT':players('#ct'),
	'T':players('#t'),
	# TODO:
	# 'WINNER'
	# 'LOSER'
	# 'WEALTHIEST'
	# 'POOREST'
	# 'TRANSFER' # connect to less full server
}

# This function is the only API to be used by the outside world.
# TODO: add variable passing
def getDetail(detail):
	try:
		value = details[detail]
		if type(value) == type(''):
			return value
		else: # we have a thunk waiting to be executed
			return value()
	except KeyError:
		return '[{'+detail+'} N/A]'
